package io.gitlab.jfronny.libjf;

import io.gitlab.jfronny.gson.JsonElement;
import org.jetbrains.annotations.ApiStatus;

import java.util.LinkedHashSet;
import java.util.Set;
import java.util.function.BiFunction;

@ApiStatus.Internal
public class Flags {
    public static Set<StringFlag> getStringFlags(String name) {
        return getFlags(name,
                (source, value) -> new StringFlag(source, value),
                (source, value) -> new StringFlag(source, value.getAsString()));
    }

    public static Set<BooleanFlag> getBoolFlags(String name) {
        return getFlags(name,
                (source, value) -> new BooleanFlag(source, true),
                (source, value) -> new BooleanFlag(source, value.getAsBoolean()));
    }

    private static <T> Set<T> getFlags(String name, BiFunction<String, String, T> makeFromProp, BiFunction<String, JsonElement, T> makeFromMod) {
        Set<T> flags = new LinkedHashSet<>();
        String propName = "libjf." + name;
        if (System.getProperty(propName) != null)
            flags.add(makeFromProp.apply("System Property", System.getProperty(propName)));
        LoaderGlue.INSTANCE.obtainFlags(name, (id, data) -> {
            flags.add(makeFromMod.apply("Mod: " + id, data));
        });
        return flags;
    }

    public record StringFlag(String source, String value) {
    }

    public record BooleanFlag(String source, boolean value) {
    }
}
