package io.gitlab.jfronny.libjf.coprocess;

import io.gitlab.jfronny.libjf.LibJf;

import java.io.Closeable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class CoProcessManager {
    private static final List<CoProcess> coProcesses = new ArrayList<>();

    public static void initialize() {
        Runtime.getRuntime().addShutdownHook(new Thread(CoProcessManager::stop));
        start();
    }

    public static void add(CoProcess coProcess) {
        coProcesses.add(coProcess);
    }

    private static void start() {
        for (CoProcess coProcess : coProcesses) {
            coProcess.start();
        }
    }

    public static void stop() {
        for (Iterator<CoProcess> iter = coProcesses.iterator(); iter.hasNext(); ) {
            CoProcess coProcess = iter.next();
            coProcess.stop();
            if (coProcess instanceof Closeable cl) {
                try {
                    cl.close();
                } catch (IOException e) {
                    LibJf.LOGGER.error("Could not close co-process", e);
                }
            }
            iter.remove();
        }
    }
}
