package io.gitlab.jfronny.libjf.fabric;

import io.gitlab.jfronny.commons.logger.SystemLoggerPlus;
import io.gitlab.jfronny.gson.JsonElement;
import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.LoaderGlue;
import io.gitlab.jfronny.libjf.coprocess.CoProcess;
import io.gitlab.jfronny.libjf.coprocess.CoProcessManager;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.fabricmc.loader.api.ModContainer;
import net.fabricmc.loader.api.entrypoint.PreLaunchEntrypoint;
import net.fabricmc.loader.api.metadata.CustomValue;

import java.util.function.BiConsumer;

public class FabricGlue implements LoaderGlue, PreLaunchEntrypoint, ModInitializer {
    @Override
    public void onPreLaunch() {
        LibJf.setup();
    }

    @Override
    public void onInitialize() {
        for (CoProcess entrypoint : FabricLoader.getInstance().getEntrypoints(LibJf.MOD_ID + ":coprocess", CoProcess.class)) {
            CoProcessManager.add(entrypoint);
        }
        if (!LoaderGlue.INSTANCE.isClient()) ServerLifecycleEvents.SERVER_STOPPED.register(server -> CoProcessManager.stop());
        CoProcessManager.initialize();
    }

    @Override
    public void obtainFlags(String name, BiConsumer<String, JsonElement> out) {
        for (ModContainer mod : FabricLoader.getInstance().getAllMods()) {
            if (!mod.getMetadata().containsCustomValue("libjf")) continue;
            CustomValue.CvObject co = mod.getMetadata().getCustomValue("libjf").getAsObject();
            if (!co.containsKey(name)) continue;
            out.accept(mod.getMetadata().getId(), FabricLoaderGsonGenerator.toGson(co.get(name)));
        }
    }

    @Override
    public SystemLoggerPlus getEarlyLogger(String name) {
        return new LoaderPlatformLogger(name);
    }

    @Override
    public boolean isClient() {
        return FabricLoader.getInstance().getEnvironmentType() == EnvType.CLIENT;
    }
}
