package io.gitlab.jfronny.libjf.config.impl;

import io.gitlab.jfronny.commons.serialize.gson.api.v2.GsonHolders;
import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.LoaderGlue;
import io.gitlab.jfronny.libjf.config.api.v2.Category;
import io.gitlab.jfronny.libjf.config.api.v2.JfConfig;
import io.gitlab.jfronny.libjf.config.api.v2.dsl.CategoryBuilder;
import org.jetbrains.annotations.Nullable;

import java.util.LinkedList;
import java.util.List;

public class AuxiliaryMetadata {
    public static AuxiliaryMetadata of(Category category) {
        AuxiliaryMetadata meta = new AuxiliaryMetadata();
        if (category != null) {
            meta.referencedConfigs = List.of(category.referencedConfigs());
        }
        return meta.sanitize();
    }

    public static AuxiliaryMetadata of(JfConfig config) {
        AuxiliaryMetadata meta = new AuxiliaryMetadata();
        if (config != null) {
            meta.referencedConfigs = List.of(config.referencedConfigs());
        }
        return meta.sanitize();
    }

    public static @Nullable AuxiliaryMetadata forMod(String modId) {
        var metaRef = new Object() {
            AuxiliaryMetadata meta = null;
        };
        LoaderGlue.INSTANCE.obtainFlags("config", (id, data) -> {
            if (id.equals(modId)) {
                metaRef.meta = GsonHolders.API.getGson().fromJson(data, AuxiliaryMetadata.class);
            }
        });
        return metaRef.meta;
    }

    public List<String> referencedConfigs;

    public void applyTo(CategoryBuilder<?> builder) {
        if (referencedConfigs != null) referencedConfigs.forEach(builder::referenceConfig);
    }

    public AuxiliaryMetadata sanitize() {
        if (referencedConfigs == null) referencedConfigs = List.of();
        else referencedConfigs = List.copyOf(referencedConfigs);
        return this;
    }

    public AuxiliaryMetadata merge(AuxiliaryMetadata other) {
        if (other == null) return this;
        AuxiliaryMetadata meta = new AuxiliaryMetadata();
        meta.referencedConfigs = new LinkedList<>();
        meta.referencedConfigs.addAll(this.referencedConfigs);
        meta.referencedConfigs.addAll(other.referencedConfigs);
        return meta.sanitize();
    }
}
